"""Some Colander phone number validators."""
import colander

from .utils import normalize_us_phone_number


def valid_us_phone_number(node, value):
    """Checks if a phone number looks like a valid US mobile number after normalization.

    :raise: colander.Invalid on error
    """

    assert type(value) == str, "Phone number must be string type"

    normalized = normalize_us_phone_number(value)

    if not normalized.startswith("+1"):
        raise colander.Invalid(node, "Not a US phone number")

    if len(normalized) != 12:
        raise colander.Invalid(node, "Phone number doesn't have correct count of digits")






import logging

from twilio.rest import TwilioRestClient
from zope.interface import implementer

from .interfaces import ISMSService


logger = logging.getLogger(__name__)


@implementer(ISMSService)
class TwilioService:
    """Send SMS using Twilio service."""

    def __init__(self, request):
        self.request = request

    def send_sms(self, receiver, text_body, sender, log_failure):
        """Asynchronous call to Twilio.

        We execute the actual SMS sending (calling Twilio HTTP API) in the Celery worker process, so that we do not block HTTP response head. This is especially important if we send more than one SMS message per HTTP request.
        """
        registry = self.request.registry

        account = registry.settings.get("sms.twilio_account")
        token = registry.settings.get("sms.twilio_token")

        if (not account) or (not token):
            raise RuntimeError("Missing sms.twilio_account and sms.twilio_token settings")

        logger.info("Sending Twilio SMS to: %s, body: %s", receiver, text_body)

        if account and token:
            try:
                client = TwilioRestClient(account, token)
                client.messages.create(to=receiver, from_=sender, body=text_body)
            except Exception as e:
                if log_failure:
                    logger.error("Could not send SMS from %s to %s, content %s", sender, receiver, text_body)
                    logger.exception(e)
                else:
                    raise





"""Dummy SMS service for testing."""
from zope.interface import implementer

from .interfaces import ISMSService


@implementer(ISMSService)
class DummySMSService:

    last_outgoing = None
    outgoing_count = 0

    def __init__(self, request):
        self.request = request

    def send_sms(self, receiver: str, text_body: str, sender=None, log_failure=True):
        # Store sent information in global variables, so that tests can verify it
        DummySMSService.last_outgoing = text_body
        DummySMSService.outgoing_count += 1



from zope.interface import Interface


class SMSConfigurationError(RuntimeError):
    """Missing INI settings."""


class ISMSService(Interface):
    """SMS service interface definition.

    All SMS services get request object through creation.
    """

    def send_sms(receiver: str, text_body: str, sender=None, log_failure=True):
        """
        :param receiver: Receiver's phone number as international format
        :param text_body: Outbound SMS body. Usually up to 1024 characters.
        :param sender: Envelope from number. Needs to be configured in the service. If none use default configured "sms.default_from".
        :param log_failure: If there is an exception from the SMS backend then log this.
        :param async: Force asynchronous operation through task subsystem. If ``None`` respect ``sms.async`` settings.
        """




from pyramid.request import Request


class SMSEvent:
    """Base class for SMS events.

    All events are fired within the web application process.
    """
    def __init__(self, request: Request, receiver: str, text_body: str, sender: str, user_dialog: bool):
        self.request = request
        self.receiver = receiver
        self.text_body = text_body
        self.sender = sender
        self.user_dialog = user_dialog


class SMSSent(SMSEvent):
    """Outbound SMS sent."""






# -*- coding: utf-8 -*-
from pyramid.interfaces import IRequest
from pyramid.util import DottedNameResolver

from .interfaces import SMSConfigurationError
from .interfaces import ISMSService


def includeme(config):
    resolver = DottedNameResolver()

    service_name = config.registry.settings.get("sms.service")
    if not service_name:
        raise SMSConfigurationError("sms.service setting missing")

    service_cls = resolver.resolve(service_name)
    if not service_cls:
        raise SMSConfigurationError("Not a valid sms.service: {}".format(service_name))

    config.registry.registerAdapter(factory=service_cls, required=(IRequest,), provided=ISMSService)





from pytest import fixture
from pyramid.httpexceptions import HTTPOk
from pyramid.interfaces import IRequest
from pyramid import testing
from webtest import TestApp

from pyramid_sms.dummy import DummySMSService
from pyramid_sms.interfaces import ISMSService
from pyramid_sms.outgoing import send_sms


def sms_view_test(request):
    """Dummy view to simulate outgoing SMS."""

    send_sms(request, "+15551234", "Test message")

    # Check DummyService recorded this
    service = request.registry.queryAdapter(request, ISMSService)
    assert service.last_outgoing == "Test message"
    return HTTPOk()


@fixture
def sms_app(request):

    config = testing.setUp()
    config.set_default_csrf_options(require_csrf=True)
    config.add_route("test-sms", "/test-sms")
    config.add_view(sms_view_test, route_name="test-sms")
    config.registry.registerAdapter(factory=DummySMSService, required=(IRequest,), provided=ISMSService)
    config.registry.settings["sms.default_sender"] = "+15551231234"
    config.registry.settings["sms.async"] = "false"

    def teardown():
        testing.tearDown()

    app = TestApp(config.make_wsgi_app())
    return app


def test_sms(sms_app: TestApp):
    resp = sms_app.get("/test-sms")
    assert resp.status_code == 200




"""Outgoing SMS API."""

import logging
import pkg_resources

from pyramid.renderers import render
from pyramid.settings import asbool

try:
    pkg_resources.get_distribution('websauna')
    from websauna.system.http import Request
    from websauna.system.task.celery import celery_app as celery
    from websauna.system.task import RequestAwareTask
    HAS_WEBSAUNA = False
except pkg_resources.DistributionNotFound:
    from pyramid.request import Request
    HAS_WEBSAUNA = False

from .interfaces import ISMSService
from .interfaces import SMSConfigurationError
from .events import SMSSent

logger = logging.getLogger(__name__)


def _send_sms(request, receiver, text_body, sender, log_failure):
    """Perform actual SMS outbound operation through a configured service."""
    service = request.registry.queryAdapter(request, ISMSService)

    if not service:
        raise SMSConfigurationError("No SMS backend configured")

    service.send_sms(receiver, text_body, sender, log_failure)


if HAS_WEBSAUNA:
    @celery.task(base=RequestAwareTask)
    def _send_sms_async(request, receiver, from_, text_body, log_failure):
        """Celery task to send asynchronously."""
        _send_sms(request, receiver, from_, text_body, log_failure)


def send_sms(request: Request, receiver: str, text_body: str, sender: str=None, log_failure: bool=True, async: bool=None, user_dialog: bool=False):
    """Send outgoing SMS message using the default configured SMS service.

    Example:

    .. code-block:: python

        def test_sms_view(request):
            '''Dummy view to simulate outgoing SMS.'''
            send_sms(request, "+15551231234", "Test message")

    :param receiver: Receiver's phone number as international format. You should normalize this number from all user input before passing in. See :py:mod:`pyramid_sms.utils` for examples.
    :param text_body: Outbound SMS body. Usually up to 1600 characters.
    :param sender: Envelope from number. Needs to be configured in the service. If none use default configured "sms.default_from".
    :param log_failure: If there is an exception from the SMS backend then log this using Python logging system. Otherwise raise the error as an exception.
    :param async: Force asynchronous operation through task subsystem. If ``None`` respect ``sms.async`` settings. If the operation is asynchronous, this function returns instantly and does not block HTTP request due to slow API calls to a third party service.
    :param user_dialog: This SMS is part of a dialog with a known user. Use this flag to log messages with the user in your conversation dashboard. Set ``False`` to two-factor auth tokens and such.

    :raise SMSConfigurationError: If configuration settings are missing
    """
    if async is None:
        async = request.registry.settings.get("sms.async")
        if async is None:
            raise SMSConfigurationError("sms.async setting not defined")
        async = asbool(async)

    if sender is None:
        sender = request.registry.settings.get("sms.default_sender")
        if not sender:
            raise SMSConfigurationError("sms.default_sender not configured")

    # https://www.twilio.com/help/faq/sms/does-twilio-support-concatenated-sms-messages-or-messages-over-160-characters
    if len(text_body) >= 1600:
        logger.warn("Too long SMS: %s", text_body)

    logger.info("Queuing sending SMS to: %s, body: %s", receiver, text_body)

    # Put the actual Twilio operation async queue
    if async:
        if not HAS_WEBSAUNA:
            raise SMSConfigurationError("Async operations are only supported with Websauna framework")
        _send_sms_async.apply_async(args=(request, receiver, text_body, sender, log_failure,))
    else:
        _send_sms(request, receiver, text_body, sender, log_failure)

    request.registry.notify(SMSSent(request, receiver, text_body, sender, user_dialog))


def send_templated_sms(request: Request, template: str, context: dict, receiver: str, sender: str=None, log_failure: bool=True, async: bool=None, user_dialog: bool=False):
    """Send out a SMS that is constructed using a page template.

    Same as :py:meth:`pyramid_sms.outgoing.send_sms`, but uses templates instead of hardcoded messages.

    :param request: HTTP request
    :param template: Template name. Like ``welcome_sms.txt.jinja``.
    :param context: Dictionary passed to template rendering engine
    """
    text_body = render(template, context, request=request)
    send_sms(request, receiver, text_body, sender, log_failure, async, user_dialog)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "pyramid>=1.7a2",
]

test_requirements = [
    "pytest",
    "webtest",
    "colander"
]

extras_require = {
    'twilio': ["twilio"],
}

setup(
    name='pyramid_sms',
    version='0.1.1',
    description="SMS service framework for Pyramid",
    long_description=readme + '\n\n' + history,
    author="Mikko Ohtamaa",
    author_email='mikko@opensourcehacker.com',
    url='https://github.com/miohtama/pyramid_sms',
    packages=[
        'pyramid_sms',
    ],
    package_dir={'pyramid_sms':
                 'pyramid_sms'},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='pyramid sms',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Framework :: Pyramid',
    ],
    test_suite='tests',
    setup_requires=[
        "pytest-runner",
        # 'setuptools-git >= 0',
        # 'setuptools-git-version',
    ],
    tests_require=test_requirements,
    extras_require=extras_require,
)




def normalize_us_phone_number(number: str) -> str:
    """Clean phone number and convert is raw US phone number in international format.

    :param number: Hand typed US phone number like (555) 123-1234

    :return: Raw phone number like +15551231234
    """
    assert type(number) == str
    digits = "+0123456789"
    number = "".join([digit for digit in number if digit in digits])

    # International 00 prefix
    if number.startswith("00"):
        number = number[2:]

    # Assume US, with area code
    if not number.startswith("+"):
        if not number.startswith("1"):
            number = "1" + number
        number = "+" + number

    return number



from pyramid_sms.utils import normalize_us_phone_number


def test_normalize_phone_number():
    assert normalize_us_phone_number("555 123 1234") == "+15551231234"
    assert normalize_us_phone_number("+1 555 123 1234") == "+15551231234"
    assert normalize_us_phone_number("+1 555 (123) 1234") == "+15551231234"
    assert normalize_us_phone_number("5551231234") == "+15551231234"
    assert normalize_us_phone_number("005551231234") == "+15551231234"




import pytest

import colander

from pyramid_sms.validators import valid_us_phone_number


def test_valid_us_phone_number():
    assert valid_us_phone_number(None, "+18082303437") is None


def test_valid_us_phone_number_short():
    assert valid_us_phone_number(None, "8082303437") is None


def test_invalid_us_phone_number_foreign():
    with pytest.raises(colander.Invalid):
        assert valid_us_phone_number(None, "+358407439707") is None


def test_invalid_us_phone_number_no_area_code():
    with pytest.raises(colander.Invalid):
        assert valid_us_phone_number(None, "2303437") is None
